package de.intevation.flys.client.client.ui.minfo;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.ListGridFieldType;
import com.smartgwt.client.widgets.Button;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.form.validator.IsIntegerValidator;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;

import de.intevation.flys.client.client.ui.AbstractUIProvider;
import de.intevation.flys.client.shared.model.Data;
import de.intevation.flys.client.shared.model.DataItem;
import de.intevation.flys.client.shared.model.DataList;
import de.intevation.flys.client.shared.model.DefaultData;
import de.intevation.flys.client.shared.model.DefaultDataItem;


public class SedLoadEpochPanel
extends AbstractUIProvider
{
    protected ListGrid elements;
    private TextItem start;
    private TextItem end;

    public Canvas createWidget(DataList data) {
        HLayout input = new HLayout();
        VLayout root = new VLayout();
        VLayout grid = new VLayout();
        VLayout intFields = new VLayout();
        Button add = new Button(MSG.add_date());
        elements = new ListGrid();

        Label title = new Label(data.get(0).getDescription());
        title.setHeight("25px");

        DynamicForm form = new DynamicForm();
        form.setNumCols(4);
        start = new TextItem(MSG.from());
        start.setWidth(60);
        start.setValidators(new IsIntegerValidator());
        end = new TextItem(MSG.to());
        end.setWidth(60);
        end.setValidators(new IsIntegerValidator());
        form.setFields(start, end);
        add.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent ce) {
                String v1 = start.getValueAsString();
                String v2 = end.getValueAsString();
                //TODO: better validation.
                if (v1 == null || v2 == null) {
                    return;
                }
                try {
                    int v1i = Integer.parseInt(v1);
                    int v2i = Integer.parseInt(v2);
                }
                catch(NumberFormatException nfe) {
                    return;
                }
                Record r = new Record();
                r.setAttribute("from", v1);
                r.setAttribute("to", v2);
                elements.addData(r);
            }
        });

        Label sel = new Label(MSG.select());
        sel.setHeight(25);
        elements.setWidth(185);
        elements.setHeight(120);
        elements.setShowHeaderContextMenu(false);
        elements.setCanReorderFields(false);
        elements.setCanSort(false);
        elements.setCanEdit(false);
        ListGridField from = new ListGridField("from", MSG.from());
        ListGridField to = new ListGridField("to", MSG.to());
        from.setWidth(70);
        to.setWidth(70);

        final ListGridField removeField  =
            new ListGridField("_removeRecord", "Remove Record"){{
                setType(ListGridFieldType.ICON);
                setIcon(GWT.getHostPageBaseURL() + MSG.removeFeature());
                setCanEdit(false);
                setCanFilter(false);
                setCanSort(false);
                setCanGroupBy(false);
                setCanFreeze(false);
                setWidth(25);
        }};

        elements.addRecordClickHandler(new RecordClickHandler() {
                @Override
                public void onRecordClick(final RecordClickEvent event) {
                    // Just handle remove-clicks
                    if(!event.getField().getName().equals(removeField.getName())) {
                        return;
                    }
                    event.getViewer().removeData(event.getRecord());
                }
            });

        elements.setFields(from, to, removeField);

        intFields.addMember(form);
        intFields.addMember(add);
        grid.addMember(sel);
        grid.addMember(elements);
        input.addMember(intFields);
        input.addMember(grid);
        root.addMember(title);
        root.addMember(input);

        return root;
    }
    @Override
    public Canvas createOld(DataList dataList) {
        GWT.log("old............................");
        HLayout layout = new HLayout();
        layout.setWidth("400px");
        VLayout vLayout = new VLayout();
        vLayout.setWidth(130);
        Label label = new Label(dataList.getLabel());
        label.setWidth("200px");
        label.setHeight(25);

        List<Data> items = dataList.getAll();
        Data str = getData(items, "epochs");
        DataItem[] strItems = str.getItems();

        String[] pairs = strItems[0].getLabel().split(";");
        for (int i = 0; i < pairs.length; i++) {
            String[] vals = pairs[i].split(",");
            Label dateLabel = new Label(vals[0] + " - " + vals[1]);
            dateLabel.setHeight(20);
            vLayout.addMember(dateLabel);
        }
        Canvas back = getBackButton(dataList.getState());
        layout.addMember(label);
        layout.addMember(vLayout);
        layout.addMember(back);

        return layout;
    }

    @Override
    public Canvas create(DataList data) {
        VLayout layout = new VLayout();
        Canvas submit = getNextButton();
        Canvas widget = createWidget(data);

        layout.addMember(widget);
        layout.addMember(submit);
        return layout;
    }

    @Override
    protected Data[] getData() {
        List<Data> data = new ArrayList<Data>();

        ListGridRecord[] lgr = elements.getRecords();
        if (lgr.length == 0) {
            return new Data[0];
        }
        String d = "";
        for (int i = 0; i < lgr.length; i++) {
            Record r = (Record) lgr[i];
            d += r.getAttribute("from") + "," + r.getAttribute("to");
            d += ";";
        }

        DataItem item = new DefaultDataItem("epochs", null, d);
            data.add(new DefaultData(
                        "epochs",
                        null,
                        null,
                        new DataItem[] { item }));
        return data.toArray(new Data[data.size()]);
    }

}
